
#include <Arduino.h>
#include <radio.h>
#include <TEA5767.h>
#include <Wire.h>             // Подключить библиотеку Wire library (требуется для устройств I2C)
#include <Adafruit_GFX.h>     // Adafruit графическая библиотека
#include <Adafruit_ST7789.h>  // Adafruit библиотека для ST7789

// ST7789 TFT Соединение с модулем
#define TFT_CS    10          
#define TFT_DC     9          
#define TFT_RST    8   

#define CANAL_UP 2            // Следующий канал 
#define CANAL_DOWN 3          // Предыдущий канал 

Adafruit_ST7789 tft = Adafruit_ST7789(TFT_CS, TFT_DC, TFT_RST);

#define FIX_BAND RADIO_BAND_FM

// Станция по умолчанию с неё начинается при загрузке
#define FIX_STATION 8830        // Ретро FM 

TEA5767 radio;    // Создаём класс radio

uint8_t i_sidx = 0; // Значение массива (частота радиостанции и Название)

RADIO_FREQ preset[] = {8830,         9120,          9160,       9520,       9920,     10120,   10210,         10370,        10570,          };
char* STATION_NAME[]={"Ретро FM",   "Эхо Москвы",  "Культура", "Rock FM",  "Орфей",  "DFM",  "МонтеКарло",  "Максимум" ,  "РусскоеРадио" , };

void setup() {
  Serial.begin(57600);
  
  pinMode(CANAL_UP, INPUT_PULLUP);              // Подтягиваем кнопку к + питания. Внутренняя подтяжка
  pinMode(CANAL_DOWN, INPUT_PULLUP);            // Подтягиваем кнопку к + питания. Внутренняя подтяжка 

  tft.init(240, 240, SPI_MODE2);                //  ST7789 дисплей 240x240 px  
  tft.setRotation(2);                           // Поворот экрана  
  tft.fillScreen(ST77XX_BLACK);                 // Цвет заливки
  tft.fillRect(0, 37, tft.width(), -37, ST77XX_BLUE); // Первая строчка на Синим фоне
  tft.fillRect(0, 42, tft.width(), 50, ST77XX_RED);   // Вторая строчка на Красном фоне
  tft.fillRect(0, 95, tft.width(), 50, ST77XX_GREEN); // Третья строчка на Зелёном фоне
  tft.fillRect(0, 147, tft.width(), 50, ST77XX_WHITE);  // Третья строчка на Белом фоне
  tft.fillRect(0, 199, tft.width(), 50, ST77XX_YELLOW);  // Третья строчка на Жёлтом фоне





  // Инициализация радио
  radio.init();
  radio.debugEnable();
  radio.setBandFrequency(FIX_BAND, FIX_STATION); 
} 

void loop() {
  char s[12];

  // Переключение каналов
if (digitalRead(CANAL_UP) == LOW) {
    // next preset
    if (i_sidx < (sizeof(preset) / sizeof(RADIO_FREQ)) - 1) {
      i_sidx++; radio.setFrequency(preset[i_sidx]);STATION_NAME[i_sidx];
      tft.fillRect(0, 37, tft.width(), -37, ST77XX_BLUE); // Первая строчка на Синим фоне
      tft.fillRect(0, 42, tft.width(), 50, ST77XX_RED);   // Вторая строчка на Красном фоне
      tft.fillRect(0, 95, tft.width(), 50, ST77XX_GREEN); // Третья строчка на Зелёном фоне
      tft.fillRect(0, 147, tft.width(), 50, ST77XX_WHITE);  // Третья строчка на Белом фоне
      tft.fillRect(0, 199, tft.width(), 50, ST77XX_YELLOW);  // Третья строчка на Жёлтом фоне
      delay(50);
    }
  } else if (digitalRead(CANAL_DOWN) == LOW) {
    if (i_sidx > 0) {
      i_sidx--;
      radio.setFrequency(preset[i_sidx]);
      tft.fillRect(0, 37, tft.width(), -37, ST77XX_BLUE); // Первая строчка на Синим фоне
      tft.fillRect(0, 42, tft.width(), 50, ST77XX_RED);   // Вторая строчка на Красном фоне
      tft.fillRect(0, 95, tft.width(), 50, ST77XX_GREEN); // Третья строчка на Зелёном фоне
      tft.fillRect(0, 147, tft.width(), 50, ST77XX_WHITE);  // Третья строчка на Белом фоне
      tft.fillRect(0, 199, tft.width(), 50, ST77XX_YELLOW);  // Третья строчка на Жёлтом фоне
      delay(50);
    } 
  }

  
  tft.setTextWrap(false);                         // отключить опцию переноса текста
  tft.setTextColor(ST77XX_WHITE);                // установить цвет текста


  
  radio.formatFrequency(s, sizeof(s));
  Serial.print("Частота:"); 
  Serial.println(s);
  tft.setCursor(0, 5);                            // переместить курсор в положение (60, 0)
  tft.setTextSize(4);                             // Размер шрифта = 3
  tft.print(s);
  tft.setTextSize(4);                             // Размер шрифта = 4
  tft.setTextColor(ST77XX_GREEN);
  tft.setCursor(2, 50);                            // переместить курсор в положение (3, 20)
  tft.print(utf8rus(STATION_NAME[i_sidx]));  
  tft.setTextWrap(false);                          // включить опцию переноса текста
  tft.setCursor(3, 42);                            // переместить курсор в положение (3, 95)
  tft.setTextColor(ST77XX_RED);
  Serial.print("Канал:");
  tft.setCursor(5, 102);                            // переместить курсор в положение (3, 20)
  tft.print(utf8rus("Канал- "));
  tft.setTextSize(5);                             // Размер шрифта = 5
  tft.setCursor(155, 100);
  tft.print(i_sidx+1);  
  tft.setTextSize(3);                             // Размер шрифта = 3 
  Serial.print("Время:");
  tft.setCursor(30, 150);
  tft.setTextColor(ST77XX_BLACK);  
  tft.print("29/12/2021");
  tft.setCursor(50, 173);
  tft.setTextColor(ST77XX_BLACK);  
  tft.print("11:35:28"); 
  Serial.print("Погода:");
  tft.setTextSize(4);                             // Размер шрифта = 4 
  tft.setCursor(50, 206);
  tft.setTextColor(ST77XX_BLACK);  
  tft.print("-20");
  tft.drawCircle(130, 208, 4, ST77XX_BLACK);     // Выводим знак градуса ( ° )
  tft.drawCircle(130, 208, 6, ST77XX_BLACK);
  tft.setCursor(140, 206);
  tft.print("C");
  

//  delay(3000);
} // loop

/* Вывод русского шрифта Перевод из  UTF-8 в Windows-1251 */

String utf8rus(String source)
{
  int i,k;
  String target;
  unsigned char n;
  char m[2] = { '0', '\0' };

  k = source.length(); i = 0;

  while (i < k) {
    n = source[i]; i++;

    if (n >= 0xC0) {
      switch (n) {
        case 0xD0: {
          n = source[i]; i++;
          if (n == 0x81) { n = 0xA8; break; }
          if (n >= 0x90 && n <= 0xBF) n = n + 0x30;
          break;
        }
        case 0xD1: {
          n = source[i]; i++;
          if (n == 0x91) { n = 0xB8; break; }
          if (n >= 0x80 && n <= 0x8F) n = n + 0x70;
          break;
        }
      }
    }
    m[0] = n; target = target + String(m);
  }
return target;
}
